"""Utility functions for sending emails."""
import django
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.encoding import force_text

try:
    import mailer
except ImportError:
    pass

from .utils import html_to_plain_text


def send_email(request, extra_context, subject_template, body_template,
               from_email, recipients, priority="medium", reply_to=None,
               headers={}):
    """
    Sends an email based on templates for subject and body.

    :param request: The current request instance.
    :param extra_context: A dictionary of items that should be added to the
        templates' contexts.
    :param subject_template: A string representing the path to the template of
        of the email's subject.
    :param body_template: A string representing the path to the template of
        the email's body.
    :param from_email: String that represents the sender of the email.
    :param recipients: A list of tuples of recipients. The tuples are similar
        to the ADMINS setting.
    :param priority: Sets the priority of the email (only used by django-mailer
        to prioritise email sendings).
    :param reply_to: Optional email address to reply to.
    :param headers: Additional dictionary to add header attributes.

    """
    if reply_to:
        if django.get_version() >= '1.8':
            # The reply_to argument has been added in 1.8
            reply_to = [reply_to]
        else:
            headers.update({'Reply-To': reply_to})
    if request:
        context = RequestContext(request, extra_context)
    else:
        context = extra_context
    if request and request.get_host():
        domain = request.get_host()
        protocol = 'https://' if request.is_secure() else 'http://'
    else:
        domain = getattr(settings, 'DOMAIN', Site.objects.get_current().domain)
        protocol = getattr(settings, 'PROTOCOL', 'http://')
    context.update({
        'domain': domain,
        'protocol': protocol,
    })
    subject = render_to_string(subject_template, context)
    subject = ''.join(subject.splitlines())
    message_html = render_to_string(body_template, context)
    message_plaintext = html_to_plain_text(message_html)
    if settings.EMAIL_BACKEND == 'mailer.backend.DbBackend':
        mailer.send_html_mail(
            subject, message_plaintext, message_html, from_email, recipients,
            priority=priority, headers=headers)
    else:
        subject = force_text(subject)
        message = force_text(message_plaintext)

        if django.get_version() >= '1.8':
            # The reply_to argument has been added in 1.8
            email = EmailMessage(
                subject=subject, body=message, from_email=from_email,
                to=recipients, headers=headers, reply_to=reply_to)
            email = EmailMultiAlternatives(
                email.subject, email.body, email.from_email, email.to,
                headers=email.extra_headers, reply_to=reply_to)
        else:
            email = EmailMessage(
                subject=subject, body=message, from_email=from_email,
                to=recipients, headers=headers)
            email = EmailMultiAlternatives(
                email.subject, email.body, email.from_email, email.to,
                headers=email.extra_headers)
        email.attach_alternative(message_html, "text/html")
        email.send()



"""Useful defualt settings that you might want to use in all your projects."""
import re


IGNORABLE_404_URLS = [
    re.compile(r'\..*$'),
    re.compile(r'^/media/'),
    re.compile(r'^/static/'),
]


IGNORABLE_404_USER_AGENTS = [
    re.compile(r'AhrefsBot', re.I),
    re.compile(r'BacklinkCrawler', re.I),
    re.compile(r'Baiduspider', re.I),
    re.compile(r'bingbot', re.I),
    re.compile(r'BLEXBot', re.I),
    re.compile(r'Cliqzbot', re.I),
    re.compile(r'coccoc', re.I),
    re.compile(r'DotBot', re.I),
    re.compile(r'EasouSpider', re.I),
    re.compile(r'Exabot', re.I),
    re.compile(r'FacebookBot', re.I),
    re.compile(r'Feedfetcher-Google', re.I),
    re.compile(r'Googlebot', re.I),
    re.compile(r'Jorgee', re.I),
    re.compile(r'Mail.RU_Bot', re.I),
    re.compile(r'mindUpBot', re.I),
    re.compile(r'MJ12bot', re.I),
    re.compile(r'msnbot', re.I),
    re.compile(r'publiclibraryarchive.org', re.I),
    re.compile(r'RU_Bot', re.I),
    re.compile(r'savetheworldheritage.org', re.I),
    re.compile(r'seoscanners', re.I),
    re.compile(r'Test Certificate Info', re.I),
    re.compile(r'Twitterbot', re.I),
    re.compile(r'WinHttp.WinHttpRequest.5', re.I),
    re.compile(r'XoviBot', re.I),
    re.compile(r'Yahoo! Slurp', re.I),
    re.compile(r'YandexBot', re.I),
]



"""Models of the ``django_libs`` projects."""
from django.core.validators import RegexValidator
from django.db.models import CharField

try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules([], ["^django_libs\.models\.ColorField"])

from .widgets import ColorPickerWidget


class ColorField(CharField):
    """Custom color field to display a color picker."""
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 6
        super(ColorField, self).__init__(*args, **kwargs)
        self.validators.append(RegexValidator(
            regex='^([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$',
            message='Only RGB color model inputs allowed, like 00000',
            code='nomatch'))

    def formfield(self, **kwargs):
        kwargs['widget'] = ColorPickerWidget
        return super(ColorField, self).formfield(**kwargs)



"""Custom admin command to clean up mailer's messagelog."""
from django.core.management.base import BaseCommand
from django.utils.timezone import now, timedelta

from mailer.models import MessageLog


class Command(BaseCommand):
    def handle(self, **options):
        # Get logs older than 4 months
        old_logs = MessageLog.objects.filter(
            when_attempted__lt=now() - timedelta(days=122))
        if not old_logs:
            print('No message logs to delete.')
            return
        log_count = old_logs.count()
        old_logs.delete()
        print('Deleted {} message logs.'.format(log_count))



"""Tests for the view classes of ``django-libs``."""
from django.contrib.sites.models import Site
try:
    from django_comments.models import Comment
except ImportError:
    pass
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.views.generic import TemplateView, View

from .. import views
from .factories import UserFactory
from .mixins import ViewTestMixin, ViewRequestFactoryTestMixin


class HybridViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``HybridView`` view class."""
    longMessage = True

    def setUp(self):
        self.user = UserFactory()

        self.authed_view = TemplateView.as_view(template_name='base.html')
        self.authed_view_kwargs = {'authed': True}
        self.anonymous_view = TemplateView.as_view(template_name='base.html')
        self.anonymous_view_kwargs = {'anonymous': True}
        self.other_anonymous_view = View.as_view()
        self.view_kwargs = {
            'authed_view': self.authed_view,
            'authed_view_kwargs': self.authed_view_kwargs,
            'anonymous_view': self.anonymous_view,
            'anonymous_view_kwargs': self.anonymous_view_kwargs}

    def get_view_name(self):
        return self.view_name

    def test_view(self):
        self.view_name = 'dummy_hybrid'
        self.should_be_callable_when_anonymous()

        bad_kwargs = self.view_kwargs.copy()
        bad_kwargs.update({'post': 'this should not be defined here'})
        self.assertRaises(TypeError, views.HybridView.as_view, **bad_kwargs)

        bad_kwargs = self.view_kwargs.copy()
        bad_kwargs.update({'wrongattr': 'this is not defined on the view'})
        self.assertRaises(TypeError, views.HybridView.as_view, **bad_kwargs)

        self.should_be_callable_when_authenticated(self.user)


class PaginatedCommentAJAXViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``PaginatedCommentAJAXView`` view class."""
    view_class = views.PaginatedCommentAJAXView

    def setUp(self):
        self.user = UserFactory()
        self.comment = Comment.objects.create(
            content_type=ContentType.objects.get(name='user'),
            object_pk=self.user.pk,
            comment='foobar',
            site=Site.objects.create(),
        )

    def test_view(self):
        self.is_not_callable()
        self.is_callable(ajax=True)


class RapidPrototypingViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``RapidPrototypingView`` view class."""
    longMessage = True

    def setUp(self):
        self.view_class = views.RapidPrototypingView

    def get_view_name(self):
        return 'prototype'

    def get_view_kwargs(self):
        return {'template_path': 'django_libs/analytics.html'}

    def test_view(self):
        self.is_callable()


class UpdateSessionAJAXViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``UpdateSessionAJAXView`` view class."""
    longMessage = True

    def setUp(self):
        self.view_class = views.UpdateSessionAJAXView

    def get_view_name(self):
        return 'update_session'

    def test_view(self):
        self.is_forbidden()
        data = {'session_name': 'foo', 'session_value': 'bar'}
        resp = self.is_postable(ajax=True, data=data)
        self.assertEqual(resp.content, 'done')


class UpdateCookieAJAXViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``UpdateCookieAJAXView`` view class."""
    longMessage = True

    def setUp(self):
        self.view_class = views.UpdateCookieAJAXView

    def get_view_name(self):
        return 'update_cookie'

    def test_view(self):
        self.is_forbidden()
        data = {'cookie_key': 'foo', 'cookie_value': 'bar'}
        resp = self.is_postable(ajax=True, data=data)
        self.assertEqual(resp.content, 'done')



"""Custom form widgets."""
from django.conf import settings
from django.forms import widgets, TextInput
from django.forms.utils import flatatt
from django.template import Context
from django.template.loader import get_template
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy


class LibsImageWidget(widgets.ClearableFileInput):
    """
    A better file input widget.

    Remember to include the js file as well. See docs for details.

    """

    initial_text = ugettext_lazy('Currently')
    input_text = ugettext_lazy('Change')
    clear_checkbox_label = ugettext_lazy('Clear')
    template_path = 'django_libs/partials/libs_image_widget.html'

    def __init__(self, attrs=None):
        self.classes = attrs.get('class', '')
        super(LibsImageWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        checkbox_name = self.clear_checkbox_name(name)
        checkbox_id = self.clear_checkbox_id(checkbox_name)
        context = {
            'checkbox_name': conditional_escape(checkbox_name),
            'checkbox_id': conditional_escape(checkbox_id),
            'checkbox_label': self.clear_checkbox_label,
            'input_id': '#{0}'.format(final_attrs['id']),
            'initial_text': self.initial_text,
            'input_text': self.input_text,
            'classes': self.classes,
            'widget': self,
            'value': value,
            'input': mark_safe('<input{0}/>'.format(flatatt(final_attrs))),
        }

        if value is None:
            value = ''
        if value and hasattr(value, "url"):
            # Only add the 'value' attribute if a value is non-empty.
            context.update({'initial': True})
            if value.url:
                context.update({
                    'src': value.url,
                    'file_name': value.url.split('/')[-1]})
                if not self.is_required:
                    context.update({
                        'clear_widget': widgets.CheckboxInput().render(
                            checkbox_name, False, attrs={'id': checkbox_id})})

        t = get_template(self.template_path)
        c = Context(context)
        return t.render(c)


class ColorPickerWidget(TextInput):
    class Media:
        css = {
            'all': (
                settings.STATIC_URL + 'django_libs/css/colorpicker.css',
            )
        }
        js = (
            settings.STATIC_URL + 'django_libs/js/jquery-1.11.1.min.js',
            settings.STATIC_URL + 'django_libs/js/colorpicker.js',
            settings.STATIC_URL + 'django_libs/js/colorpicker_list.js',
            settings.STATIC_URL + 'django_libs/js/eye.js',
            settings.STATIC_URL + 'django_libs/js/layout.js',
            settings.STATIC_URL + 'django_libs/js/utils.js',
        )

    def __init__(self, language=None, attrs=None):
        self.language = language or settings.LANGUAGE_CODE[:2]
        super(ColorPickerWidget, self).__init__(attrs=attrs)

    def render(self, name, value, attrs=None):
        rendered = super(ColorPickerWidget, self).render(name, value, attrs)
        return rendered + mark_safe(
            u'''<script type="text/javascript">
                $('#id_%s').ColorPicker({
                onSubmit: function(hsb, hex, rgb, el) {
                    $(el).val(hex);
                    $(el).ColorPickerHide();
                },
                onBeforeShow: function () {
                    $(this).ColorPickerSetColor(this.value);
                }
             }).bind('keyup', function(){
                 $(this).ColorPickerSetColor(this.value);
             });
            </script>''' % name)



"""Tests for the model mixins of ``django_libs``."""
from django.test import TestCase
from django.utils.translation import activate

from .test_app.factories import HvadDummyFactory
from .test_app.models import HvadDummy


class TranslationModelMixinTestCase(TestCase):
    """Tests for the ``TranslationModelMixin`` model mixin."""
    longMessage = True

    def test_functions(self):
        translated_obj = HvadDummyFactory()
        self.assertEqual(translated_obj.translation_getter('title'), 'title1')
        untranslated_obj = HvadDummy()
        self.assertIsNone(untranslated_obj.translation_getter('title'))
        untranslated_obj.translate('de')
        untranslated_obj.title = 'foo'
        untranslated_obj.save()
        activate('de')
        self.assertEqual(untranslated_obj.translation_getter('title'), 'foo')
        activate('en')



"""Models for the ``test_app`` app."""
from django.db import models
from django.utils.translation import ugettext_lazy as _

from hvad.models import TranslatableModel, TranslatedFields

from ...models_mixins import TranslationModelMixin


class HvadDummy(TranslationModelMixin, TranslatableModel):
    """Dummy model to test hvad stuff."""
    translations = TranslatedFields(
        title=models.CharField(
            verbose_name=_('Title'),
            max_length=256,
        ),
    )


class DummyProfile(models.Model):
    """Just a dummy profile model for testing purposes."""
    user = models.ForeignKey('auth.User')
    dummy_field = models.CharField(
        verbose_name=_('Dummy Field'),
        max_length=128,
    )



"""Tests for the utility functions in ``loaders.py``."""
from django.test import TestCase

from ..loaders import load_member, load_member_from_setting, split_fqn


def callable_that_returns_fqn_string():
    """Dummy function Used by tests."""
    return 'django_libs.loaders.load_member'


class LoadMemberTestCase(TestCase):
    """Tests for the ``load_member`` utility function."""
    def test_function(self):
        member = load_member('django_libs.loaders.load_member')
        self.assertEqual(member.func_name, 'load_member')


class LoadMemberFromSetting(TestCase):
    """Tests for the ``load_member_from_setting`` utility function."""
    def test_function(self):
        member = load_member_from_setting('TEST_LOAD_MEMBER')
        self.assertEqual(member.func_name, 'load_member')


class SplitFqnTestCase(TestCase):
    """Tests for the ``split_fqn`` utility function."""
    def test_with_string(self):
        modulename, membername = split_fqn('django_libs.loaders.load_member')
        self.assertEqual(modulename, 'django_libs.loaders')
        self.assertEqual(membername, 'load_member')

    def test_with_callable(self):
        modulename, membername = split_fqn(callable_that_returns_fqn_string)
        self.assertEqual(modulename, 'django_libs.loaders')
        self.assertEqual(membername, 'load_member')



"""Just some factories for the test app."""
import factory

from ..factories import HvadFactoryMixin, UserFactory
from models import DummyProfile, HvadDummy


class HvadDummyFactory(HvadFactoryMixin, factory.DjangoModelFactory):
    """Factory for the ``HvadDummy`` model."""
    title = factory.Sequence(lambda n: 'title{}'.format(n))

    class Meta:
        model = HvadDummy


class DummyProfileFactory(factory.DjangoModelFactory):
    """Factory for the ``DummyProfile`` model."""
    user = factory.SubFactory(UserFactory)
    dummy_field = factory.Sequence(lambda n: 'dummyfield{}'.format(n))

    class Meta:
        model = DummyProfile



"""Custom filters for the ``compressor`` app."""
from django.conf import settings

from compressor.filters.css_default import CssAbsoluteFilter
from compressor.utils import staticfiles


class S3CssAbsoluteFilter(CssAbsoluteFilter):
    """
    This CSS filter was built to use django-compressor in combination with a
    Amazon S3 storage. It will make sure to provide the right URLs, whether
    you're in DEBUG mode or not.

    Make sure to add the ``FULL_DOMAIN`` setting. This is your base url, e.g.
    'https://www.example.com'.

    """
    def __init__(self, *args, **kwargs):
        super(S3CssAbsoluteFilter, self).__init__(*args, **kwargs)
        self.url = '%s%s' % (settings.FULL_DOMAIN, settings.STATIC_URL)
        self.url_path = self.url

    def find(self, basename):
        # The line below is the original line.  I removed settings.DEBUG.
        # if settings.DEBUG and basename and staticfiles.finders:
        if basename and staticfiles.finders:
            return staticfiles.finders.find(basename)



"""Logging related utilities."""
import logging

from django.conf import settings


class AddCurrentUser(logging.Filter):
    """
    Adds the current user to the log record.

    See docs for usage.

    """
    def filter(self, record):
        if hasattr(record.request.user, 'email'):
            record.request.META['CURRENT_USER'] = record.request.user.email
        return True


class FilterIgnorable404URLs(logging.Filter):
    """
    Takes the IGNORABLE_404_URLS setting and disables logging for mathing URLs.

    You can also set IGNORABLE_404_USER_AGENTS to disable logging for matching
    user agents (for example to ignore certain bots).

    Add it to your logging filters like so::

        'filters': {
            'require_debug_false': {
                '()': 'django.utils.log.RequireDebugFalse'
            },
            'filter_ignorable_404_urls': {
                '()': 'django_libs.utils_log.FilterIgnorable404URLs'
            }
        },

    Apply the filter to your handlers like so::

        'handlers': {
            'mail_admins': {
                'level': 'WARNING',
                'filters': ['require_debug_false', filter_ignorable_404_urls', ],  # NOQA
                'class': 'django.utils.log.AdminEmailHandler'
            },

    """
    def filter(self, record):
        request = record.request

        if record.status_code != 404:
            # No 404? No business for this filter.
            return True

        user_agent = request.META.get('HTTP_USER_AGENT')
        if getattr(settings, 'IGNORABLE_404_USER_AGENTS'):
            is_ignorable = any(
                pattern.search(user_agent)
                for pattern in settings.IGNORABLE_404_USER_AGENTS)
            if is_ignorable:
                return False

        path = request.get_full_path()
        if getattr(settings, 'IGNORABLE_404_URLS'):
            is_ignorable = any(
                pattern.search(path)
                for pattern in settings.IGNORABLE_404_URLS)
            if is_ignorable:
                return False

        return True



"""Tests for the widgets of the ``django_libs`` app."""
from django.test import TestCase

from ..widgets import ColorPickerWidget


class ColorPickerWidgetTestCase(TestCase):
    """Tests for the ``ColorPickerWidget`` widget."""
    longMessage = True

    def setUp(self):
        self.widget = ColorPickerWidget()

    def test_render_tag(self):
        self.assertIn('value="ffffff"', self.widget.render('field', 'ffffff'),
                      msg=('Should render the input form.'))



"""Custom middleware for Django 1.6 projects."""
import re

from django.conf import settings
from django.core.mail import mail_managers
from django.utils.encoding import force_text


class CustomBrokenLinkEmailsMiddleware(object):
    """Custom version that adds the user to the error email."""
    def process_response(self, request, response):
        """
        Send broken link emails for relevant 404 NOT FOUND responses.
        """
        if response.status_code == 404 and not settings.DEBUG:
            domain = request.get_host()
            path = request.get_full_path()
            referer = force_text(
                request.META.get('HTTP_REFERER', ''), errors='replace')

            if not self.is_ignorable_request(request, path, domain, referer):
                ua = request.META.get('HTTP_USER_AGENT', '<none>')
                ip = request.META.get('REMOTE_ADDR', '<none>')

                user = None
                if request.user and hasattr(request.user, 'email'):
                    user = request.user.email
                content = (
                    "Referrer: %s\n"
                    "Requested URL: %s\n"
                    "User agent: %s\n"
                    "IP address: %s\n"
                    "User: %s\n"
                ) % (referer, path, ua, ip, user)
                if self.is_internal_request(domain, referer):
                    internal = 'INTERNAL '
                else:
                    internal = ''
                mail_managers(
                    "Broken %slink on %s" % (
                        internal,
                        domain
                    ),
                    content,
                    fail_silently=True)
        return response

    def is_internal_request(self, domain, referer):
        """
        Returns True if referring URL is the same domain as current request.

        """
        # Different subdomains are treated as different domains.
        return bool(re.match("^https?://%s/" % re.escape(domain), referer))

    def is_ignorable_request(self, request, uri, domain, referer):
        """
        Returns True if the given request *shouldn't* notify the site managers.
        """
        # '?' in referer is identified as search engine source
        if (not referer or
                (not self.is_internal_request(domain, referer)
                    and '?' in referer)):
            return True
        return any(
            pattern.search(uri) for pattern in settings.IGNORABLE_404_URLS)



"""Tests for the forms utilities of django_libs."""
from django import forms
from django.test import TestCase

from .. import forms as libs_forms


class DummyForm(libs_forms.StripTagsFormMixin, forms.Form):
    text = forms.CharField(max_length=1024)

    STRIP_TAGS_FIELDS = ['text', ]

    def __init__(self, *args, **kwargs):
        super(DummyForm, self).__init__(*args, **kwargs)
        self.strip_tags()


class StripTagsFormMixinTestCase(TestCase):
    """Tests for the ``StripTagsFormMixin``."""
    longMessage = True

    def test_mixin(self):
        form = DummyForm(data={'text': '<em>Foo</em>'})
        self.assertEqual(form.data['text'], 'Foo', msg=(
            'The mixin should strip away the tags from the given form data'))

        form = DummyForm(prefix='bar', data={'bar-text': '<em>Foo</em>'})
        self.assertEqual(form.data['bar-text'], 'Foo', msg=(
            'The mixin still works when the form has a prefix'))



"""Views for testing 404 and 500 templates."""
import json
import datetime
import math
from functools import update_wrapper

from django.conf import settings
from django.core.paginator import EmptyPage, PageNotAnInteger, Paginator
try:
    from django_comments.models import Comment
except ImportError:
    pass
from django.http import Http404, HttpResponse, HttpResponseForbidden
from django.template import loader, Context
from django.views.generic import TemplateView, View

from . import default_settings


class Http404TestView(TemplateView):
    """
    WARNING: This view is deprecated. Use the ``RapidPrototypingView`` instead.

    """
    template_name = '404.html'


class Http500TestView(TemplateView):
    """
    WARNING: This view is deprecated. Use the ``RapidPrototypingView`` instead.

    """
    template_name = '500.html'


class HybridView(View):
    """
    View that renders different views depending on wether the user is authed.

    If the user is authenticated, it will render ``authed_view``, otherwise
    it will render ``anonymous_view``.

    If you are passing in a function based view you can also define
    ``authed_view_kwargs`` and ``anonymous_view_kwargs`` which, of course,
    should be dictionaries.

    """
    authed_view = None
    authed_view_kwargs = None
    anonymous_view = None
    anonymous_view_kwargs = None

    @classmethod
    def as_view(cls, **initkwargs):
        """
        Main entry point for a request-response process.
        """
        # sanitize keyword arguments
        for key in initkwargs:
            if key in cls.http_method_names:
                raise TypeError("You tried to pass in the %s method name as a "
                                "keyword argument to %s(). Don't do that."
                                % (key, cls.__name__))
            if not hasattr(cls, key):
                raise TypeError("%s() received an invalid keyword %r. as_view "
                                "only accepts arguments that are already "
                                "attributes of the class." % (
                                    cls.__name__, key))

        def view(request, *args, **kwargs):
            self = cls(**initkwargs)
            if hasattr(self, 'get') and not hasattr(self, 'head'):
                self.head = self.get
            self.request = request
            self.args = args
            self.kwargs = kwargs
            self.authed_view = initkwargs.get('authed_view')
            self.authed_view_kwargs = initkwargs.get('authed_view_kwargs')
            self.anonymous_view = initkwargs.get('anonymous_view')
            self.anonymous_view_kwargs = initkwargs.get(
                'anonymous_view_kwargs')
            return self.dispatch(request, *args, **kwargs)
        # take name and docstring from class
        update_wrapper(view, cls, updated=())
        # and possible attributes set by decorators
        # like csrf_exempt from dispatch
        update_wrapper(view, cls.dispatch, assigned=())
        return view

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated():
            view_kwargs = self.authed_view_kwargs or {}
            return self.authed_view(request, **view_kwargs)

        view_kwargs = self.anonymous_view_kwargs or {}
        return self.anonymous_view(request, **view_kwargs)


class PaginatedCommentAJAXView(TemplateView):
    """Returns a page of comments for an object."""
    template_name = 'django_libs/partials/ajax_comments.html'

    def dispatch(self, request, *args, **kwargs):
        if not request.is_ajax():
            raise Http404

        self.comments = Comment.objects.filter(
            content_type__name=request.GET.get('ctype'),
            object_pk=request.GET.get('object_pk'))
        return super(PaginatedCommentAJAXView, self).dispatch(
            request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        template = loader.get_template(self.template_name)
        content = template.render(Context(self.get_context_data()))
        return HttpResponse(json.dumps({
            'data': content, 'page': self.page,
            'has_prev': self.page_obj.has_previous(),
            'has_next': self.page_obj.has_next()}),
            content_type="application/json")

    def get_context_data(self):
        ctx = super(PaginatedCommentAJAXView, self).get_context_data()

        # Let's try to figure out if a special comment was requested
        page = None
        comment_pk = self.request.GET.get('comment_pk')
        if comment_pk:
            try:
                self.comment = Comment.objects.get(
                    pk=comment_pk, is_public=True, is_removed=False)
            except Comment.DoesNotExist:
                self.comment = None
            if self.comment:
                index = 1
                for item in self.comments:
                    if item == self.comment:
                        # The special comment is indeed part of all comments,
                        # so let's calculate the page it should be on
                        page = math.ceil(
                            float(index) /
                            default_settings.COMMENTS_PAGINATE_BY)
                    index += 1

        if page:
            # If we had found a special comment, we ignore the ?page param
            # and jump to that comment's page instead
            self.page = page
        else:
            self.page = self.request.GET.get('page')

        # Now we know which page we want to show and we can start the paginator
        self.paginator = Paginator(
            self.comments, default_settings.COMMENTS_PAGINATE_BY)
        try:
            self.page_obj = self.paginator.page(self.page)
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            self.page_obj = self.paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999),
            # deliver last page of results.
            self.page_obj = self.paginator.page(self.paginator.num_pages)

        ctx.update({
            'comments': self.comments,
            'paginator': self.paginator,
            'page_obj': self.page_obj,
        })
        return ctx


class RapidPrototypingView(TemplateView):
    """
    View that can render any given template.

    This can be useful when you want your designers to be bale to go ahead and
    create templates although no views have been created for those templates,
    yet.

    """
    def dispatch(self, request, *args, **kwargs):
        self.template_name = kwargs.get('template_path')
        return super(RapidPrototypingView, self).dispatch(request, *args,
                                                          **kwargs)


class UpdateSessionAJAXView(View):
    """View to update a session variable in an AJAX post."""
    def dispatch(self, request, *args, **kwargs):
        if not (request.is_ajax() and request.method == 'POST'):
            return HttpResponseForbidden()
        if (request.POST.get('session_name')
                and request.POST.get('session_value')):
            request.session[request.POST['session_name']] = request.POST[
                'session_value']
        return HttpResponse('done')


class UpdateCookieAJAXView(View):
    """View to update a cookie in an AJAX post."""
    def dispatch(self, request, *args, **kwargs):
        if not (request.is_ajax() and request.method == 'POST'):
            return HttpResponseForbidden()
        response = HttpResponse('done')
        days = int(request.POST.get('cookie_days', 100))
        date = datetime.datetime.utcnow() + datetime.timedelta(days=days)
        expires = datetime.datetime.strftime(date, "%a, %d-%b-%Y %H:%M:%S GMT")
        response.set_cookie(
            request.POST['cookie_key'],
            request.POST['cookie_value'],
            max_age=(days * 24 * 60 * 60),
            expires=expires,
            domain=settings.SESSION_COOKIE_DOMAIN,
            secure=settings.SESSION_COOKIE_SECURE or None,
        )
        return response



"""Tests for the models of the ``django_libs`` app."""
from django.test import TestCase

from ..models import ColorField
from ..widgets import ColorPickerWidget


class ColorFieldTestCase(TestCase):
    """Tests for the ``ColorField`` model."""
    longMessage = True

    def test_functions(self):
        color_field = ColorField()
        color_field.formfield
        self.assertIsInstance(
            color_field.formfield().widget, ColorPickerWidget, msg=(
                'Should add the color field widget.'))



"""
IMPORTANT: The following factories are still available, but no longer
maintained. We recommend to use https://github.com/klen/mixer for fixtures.

"""
from io import BytesIO

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.files.uploadedfile import SimpleUploadedFile
from django.utils.timezone import now

from PIL import Image
from hashlib import md5
import factory


class HvadFactoryMixin(object):
    """
    Overrides ``_create`` and takes care of creating a translation.

    """
    @classmethod
    def _create(cls, target_class, *args, **kwargs):
        obj = target_class(*args, **kwargs)

        # Factory boy and hvad behave a bit weird. When getting the object,
        # obj.some_translatable_field is actually set although no translation
        # object exists, yet. We have to cache the translatable values ...
        cached_values = {}
        for field in obj._translated_field_names:
            if field in ['id', 'master', 'master_id', 'language_code']:
                continue
            cached_values[field] = getattr(obj, field)

        # ... because when calling translate, the translatable values will be
        # lost on the obj ...
        obj.translate(obj.language_code)
        for field in obj._translated_field_names:
            if field in ['id', 'master', 'master_id', 'language_code']:
                continue
            # ... so here we will put them back on the object, this time they
            # will be saved on the translatable object.
            setattr(obj, field, cached_values[field])

        obj.save()
        return obj


class UploadedImageFactory(object):
    """Creates an uploaded image for testing."""
    def __new__(cls, **kwargs):
        return cls._create_image(**kwargs)

    @classmethod
    def _create_image(self, image_format='JPEG', filename='img.jpg'):
        thumb = Image.new('RGB', (100, 100), 'blue')
        thumb_io = BytesIO()
        thumb.save(thumb_io, format=image_format)
        self._image = SimpleUploadedFile(filename, thumb_io.getvalue())
        return self._image


class UserFactory(factory.DjangoModelFactory):
    """
    Creates a new ``User`` object.

    We use ``django-registration-email`` which allows users to sign in with
    their email instead of a username. Since the username field is too short
    for most emails, we don't really use the username field at all and just
    store a md5 hash in there.

    Username will be a random 30 character md5 value.
    Email will be ``userN@example.com`` with ``N`` being a counter.
    Password will be ``test123`` by default.

    """
    username = factory.Sequence(lambda n: md5(str(n)).hexdigest()[0:30])
    email = factory.Sequence(lambda n: 'user{0}@example.com'.format(n))

    class Meta:
        model = User

    @classmethod
    def _prepare(cls, create, **kwargs):
        password = 'test123'
        if 'password' in kwargs:
            password = kwargs.pop('password')
        user = super(UserFactory, cls)._prepare(create, **kwargs)
        user.set_password(password)
        if create:
            user.save()
        return user


class SiteFactory(factory.DjangoModelFactory):
    """
    Creates a new ``Site`` object.

    """
    name = factory.Sequence(lambda n: 'Example {}'.format(n))
    domain = factory.Sequence(lambda n: 'example{}.com'.format(n))

    class Meta:
        model = Site


try:
    from mailer.models import MessageLog
except ImportError:  # mailer not installed
    pass
else:
    class MessageLogFactory(factory.DjangoModelFactory):
        """
        Creates a new ``MessageLog`` object.

        We only use this factory for testing purposes (management command:
        "cleanup_mailer_messagelog").

        """
        message_data = 'foo'
        when_added = factory.Sequence(lambda n: now())
        priority = '3'
        result = '1'
        log_message = 'foo'

        class Meta:
            model = MessageLog



"""Useful mixins for models."""
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import get_language

from hvad.models import NoTranslation, TranslationManager


class HvadPublishedManager(TranslationManager):
    """
    Returns all objects, which are published and in the currently
    active language if check_language is True (default).

    :param request: A Request instance.
    :param check_language: Option to disable language filtering.

    """
    def published(self, request, check_language=True):
        kwargs = {'translations__is_published': True}
        if check_language:
            language = getattr(request, 'LANGUAGE_CODE', get_language())
            if not language:
                self.model.objects.none()
            kwargs.update({'translations__language_code': language})
        return self.get_query_set().filter(**kwargs)


class TranslationModelMixin(object):
    """Mixin to provide custom django-hvad overrides."""
    def __unicode__(self):
        return self.translation_getter('title')

    def check_for_result(self, language, translation, result, name):
        if not result:
            try:
                translation = self.translations.get(language_code=language)
            except ObjectDoesNotExist:
                pass
            else:  # pragma: nocover
                result = getattr(translation, name, '')
        return translation, result

    def translation_getter(self, name):
        """
        Custom translation getter.

        1. Try the current language
        2. Try base language (e.g. 'en' instead of 'en-gb')
        3. Try default sys language
        4. Use any translation

        """
        stuff = self.safe_translation_getter(name, NoTranslation)
        if stuff is not NoTranslation:
            return stuff

        translation = None
        result = None

        # Check for the current language
        translation, result = self.check_for_result(
            get_language(), translation, result, name)

        # Check for the current main language
        translation, result = self.check_for_result(
            get_language()[:2], translation, result, name)

        # Check for the default language
        translation, result = self.check_for_result(
            settings.LANGUAGE_CODE, translation, result, name)

        if not result:
            # Check for any available language
            for trans in self.translations.all():  # pragma: nocover
                translation = trans
                result = getattr(translation, name, '')
                if result:
                    break

        setattr(self, self._meta.translations_cache, translation)
        return result



"""Tests for the management commands of the ``django_libs`` app."""
from django.core.management import call_command
from django.test import TestCase
from django.utils.timezone import now, timedelta

from mailer.models import MessageLog

from .factories import MessageLogFactory


class CleanupMailerMessagelogTestCase(TestCase):
    """Tests for the ``cleanup_mailer_messagelog`` management command."""
    longMessage = True

    def test_reminder(self):
        self.assertFalse(call_command('cleanup_mailer_messagelog'))
        self.assertEqual(MessageLog.objects.all().count(), 0)
        MessageLogFactory()
        self.assertFalse(call_command('cleanup_mailer_messagelog'))
        self.assertEqual(MessageLog.objects.all().count(), 1)
        MessageLogFactory(when_attempted=now() - timedelta(days=200))
        self.assertEqual(MessageLog.objects.all().count(), 2)
        self.assertFalse(call_command('cleanup_mailer_messagelog'))
        self.assertEqual(MessageLog.objects.all().count(), 1)



"""Templatetags for the ``django_libs`` project."""
import datetime
import importlib

from django import template
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.urlresolvers import resolve, Resolver404
from django.db.models.fields import FieldDoesNotExist
from django.template.defaultfilters import truncatewords_html, stringfilter
from django.utils.text import force_unicode

from django_libs import utils

from ..loaders import load_member


register = template.Library()


@register.assignment_tag
def add_form_widget_attr(field, attr_name, attr_value, replace=0):
    """
    Adds widget attributes to a bound form field.

    This is helpful if you would like to add a certain class to all your forms
    (i.e. `form-control` to all form fields when you are using Bootstrap)::

        {% load libs_tags %}
        {% for field in form.fields %}
            {% add_form_widget_attr field 'class' 'form-control' as field_ %}
            {{ field_ }}
        {% endfor %}

    The tag will check if the attr already exists and only append your value.
    If you would like to replace existing attrs, set `replace=1`::

        {% add_form_widget_attr field 'class' 'form-control' replace=1 as
          field_ %}


    """
    if not replace:
        attr = field.field.widget.attrs.get(attr_name, '')
        attr += force_unicode(attr_value)
        field.field.widget.attrs[attr_name] = attr
        return field
    else:
        field.field.widget.attrs[attr_name] = attr_value
        return field


@register.tag('block_anyfilter')
def block_anyfilter(parser, token):
    """
    Turn any template filter into a blocktag.

    Usage::

    {% load libs_tags %}
    {% block_anyfilter django.template.defaultfilters.truncatewords_html 15 %}
        // Something complex that generates html output
    {% endblockanyfilter %}

    """
    bits = token.contents.split()
    nodelist = parser.parse(('endblockanyfilter',))
    parser.delete_first_token()
    return BlockAnyFilterNode(nodelist, bits[1], *bits[2:])


class BlockAnyFilterNode(template.Node):
    def __init__(self, nodelist, original_tag_fqn, *args):
        self.nodelist = nodelist
        self.original_tag = load_member(original_tag_fqn)
        self.args = args

    def render(self, context):
        output = self.nodelist.render(context)
        return self.original_tag(output, *self.args)


@register.tag('block_truncatewords_html')
def block_truncatewords_html(parser, token):
    """
    DEPRECATED: Use block_anyfilter instead!

    Allows to truncate any block of content.

    This is useful when rendering other tags that generate content,
    such as django-cms' ``render_placeholder`` tag, which is not available
    as an assignment tag::

        {% load libs_tags %}
        {% block_truncatewords_html 15 %}
            {% render_placeholder object.placeholder %}
        {% endblocktruncatewordshtml %}

    """
    bits = token.contents.split()
    try:
        word_count = bits[1]
    except IndexError:
        word_count = 15
    nodelist = parser.parse(('endblocktruncatewordshtml',))
    parser.delete_first_token()
    return BlockTruncateWordsHtmlNode(nodelist, word_count)


class BlockTruncateWordsHtmlNode(template.Node):
    def __init__(self, nodelist, word_count):
        self.nodelist = nodelist
        self.word_count = word_count

    def render(self, context):
        output = self.nodelist.render(context)
        return truncatewords_html(output, self.word_count)


@register.assignment_tag
def calculate_dimensions(image, long_side, short_side):
    """Returns the thumbnail dimensions depending on the images format."""
    if image.width >= image.height:
        return '{0}x{1}'.format(long_side, short_side)
    return '{0}x{1}'.format(short_side, long_side)


@register.assignment_tag
def call(obj, method, *args, **kwargs):
    """
    Allows to call any method of any object with parameters.

    Because come on! It's bloody stupid that Django's templating engine doesn't
    allow that.

    Usage::

        {% call myobj 'mymethod' myvar foobar=myvar2 as result %}
        {% call myobj 'mydict' 'mykey' as result %}
        {% call myobj 'myattribute' as result %}

    :param obj: The object which has the method that you would like to call
    :param method: A string representing the attribute on the object that
      should be called.

    """
    function_or_dict_or_member = getattr(obj, method)
    if callable(function_or_dict_or_member):
        # If it is a function, let's call it
        return function_or_dict_or_member(*args, **kwargs)
    if not len(args):
        # If it is a member, lets return it
        return function_or_dict_or_member
    # If it is a dict, let's access one of it's keys
    return function_or_dict_or_member[args[0]]


@register.assignment_tag
def concatenate(*args, **kwargs):
    """
    Concatenates the given strings.

    Usage::

        {% load libs_tags %}
        {% concatenate "foo" "bar" as new_string %}
        {% concatenate "foo" "bar" divider="_" as another_string %}

    The above would result in the strings "foobar" and "foo_bar".

    """
    divider = kwargs.get('divider', '')
    result = ''
    for arg in args:
        if result == '':
            result += arg
        else:
            result += '{0}{1}'.format(divider, arg)
    return result


@register.filter
def get_content_type(obj, field_name=False):
    """
    Returns the content type of an object.

    :param obj: A model instance.
    :param field_name: Field of the object to return.

    """
    content_type = ContentType.objects.get_for_model(obj)
    if field_name:
        return getattr(content_type, field_name, '')
    return content_type


@register.assignment_tag
def get_form_field_type(field):
    """
    Returns the widget type of the given form field.

    This can be helpful if you want to render form fields in your own way
    (i.e. following Bootstrap standards).

    Usage::

        {% load libs_tags %}
        {% for field in form %}
            {% get_form_field_type field as field_type %}
            {% if "CheckboxInput" in field_type %}
                <div class="checkbox">
                    <label>
                        // render input here
                    </label>
                </div>
            {% else %}
                {{ field }}
            {% endif %}
        {% endfor %}

    """
    return field.field.widget.__str__()


@register.filter
def get_verbose(obj, field_name=""):
    """
    Returns the verbose name of an object's field.

    :param obj: A model instance.
    :param field_name: The requested field value in string format.

    """
    if hasattr(obj, "_meta") and hasattr(obj._meta, "get_field_by_name"):
        try:
            return obj._meta.get_field_by_name(field_name)[0].verbose_name
        except FieldDoesNotExist:
            pass
    return ""


@register.assignment_tag
def get_profile_for(user):
    """
    Allows to call the get_profile utility function from django-libs in a
    template.

    """
    return utils.get_profile(user)


@register.assignment_tag
def get_query_params(request, *args):
    """
    Allows to change one of the URL get parameter while keeping all the others.

    Usage::

      {% load libs_tags %}
      {% get_query_params request "page" page_obj.next_page_number as query %}
      <a href="?{{ query }}">Next</a>

    You can also pass in several pairs of keys and values::

      {% get_query_params request "page" 1 "foobar" 2 as query %}

    You often need this when you have a paginated set of objects with filters.

    Your url would look something like ``/?region=1&gender=m``. Your paginator
    needs to create links with ``&page=2`` in them but you must keep the
    filter values when switching pages.

    :param request: The request instance.
    :param *args: Make sure to always pass in paris of args. One is the key,
      one is the value. If you set the value of a key to "!remove" that
      parameter will not be included in the returned query.

    """
    query = request.GET.copy()
    index = 1
    key = ''
    for arg in args:
        if index % 2 != 0:
            key = arg
        else:
            if arg == "!remove":
                try:
                    query.pop(key)
                except KeyError:
                    pass
            else:
                query[key] = arg
        index += 1
    return query.urlencode()


class LoadContextNode(template.Node):
    def __init__(self, fqn):
        self.fqn = fqn

    def render(self, context):
        module = importlib.import_module(self.fqn)
        for attr in dir(module):
            if not attr.startswith('__'):
                context[attr] = getattr(module, attr)
        return ''


@register.tag
def load_context(parser, token):
    # TODO Docstring!
    try:
        # split_contents() knows not to split quoted strings.
        tag_name, fqn = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            '%r tag requires a single argument' % token.contents.split()[0])
    if not (fqn[0] == fqn[-1] and fqn[0] in ('"', "'")):
        raise template.TemplateSyntaxError(
            "%r tag's argument should be in quotes" % tag_name)
    return LoadContextNode(fqn[1:-1])


@register.simple_tag
def navactive(request, url, exact=0, use_resolver=1):
    """
    Returns ``active`` if the given URL is in the url path, otherwise ''.

    Usage::

        {% load libs_tags %}
        ...
        <li class="{% navactive request "/news/" exact=1 %}">

    :param request: A request instance.
    :param url: A string representing a part of the URL that needs to exist
      in order for this method to return ``True``.
    :param exact: If ``1`` then the parameter ``url`` must be equal to
      ``request.path``, otherwise the parameter ``url`` can just be a part of
      ``request.path``.
    :use_resolver: If ``0`` we will not try to compare ``url`` with existing
      view names but we will only compare it with ``request.path``.

    """
    if use_resolver:
        try:
            if url == resolve(request.path).url_name:
                # Checks the url pattern in case a view_name is posted
                return 'active'
            elif url == request.path:
                # Workaround to catch URLs with more than one part, which don't
                # raise a Resolver404 (e.g. '/index/info/')
                match = request.path
            else:
                return ''
        except Resolver404:
            # Indicates, that a simple url string is used (e.g. '/index/')
            match = request.path
    else:
        match = request.path

    if exact and url == match:
        return 'active'
    elif not exact and url in request.path:
        return 'active'
    return ''


@register.filter
def get_range(value, max_num=None):
    """
    Returns the range over a given value.

    Usage::

        {% load libs_tags %}

        {% for item in object_list.count|get_range %}
            {{ item }} // render real items here
        {% endfor %}
        {% for item in object_list.count|get_range:5 %}
            // render placeholder items here
        {% endfor %}

    :param value: The number to pass to the range function
    :param max_num: Optional. Use this if you want to get a range over the
      difference between the actual number and a maximum amount. This can
      be useful to display placeholder items in a situation where the
      space must always be filled up with 5 items but your actual list
      might only have 2 items.

    """
    if max_num:
        value = max_num - value
    return range(value)


@register.assignment_tag
def get_range_around(range_value, current_item, padding):
    """
    Returns a range of numbers around the given number.

    This is useful for pagination, where you might want to show something
    like this::

        << < ... 4 5 (6) 7 8 .. > >>

    In this example `6` would be the current page and we show 2 items around
    that page (including the page itself).

    Usage::

        {% load libs_tags %}
        {% get_range_around page_obj.paginator.num_pages page_obj.number 5
          as pages %}

    :param range_amount: Number of total items in your range (1 indexed)
    :param current_item: The item around which the result should be centered
      (1 indexed)
    :param padding: Number of items to show left and right from the current
      item.

    """
    total_items = 1 + padding * 2
    left_bound = padding
    right_bound = range_value - padding
    if range_value <= total_items:
        range_items = range(1, range_value+1)
        return {
            'range_items': range_items,
            'left_padding': False,
            'right_padding': False,
        }
    if current_item <= left_bound:
        range_items = range(1, range_value+1)[:total_items]
        return {
            'range_items': range_items,
            'left_padding': range_items[0] > 1,
            'right_padding': range_items[-1] < range_value,
        }

    if current_item >= right_bound:
        range_items = range(1, range_value+1)[-total_items:]
        return {
            'range_items': range_items,
            'left_padding': range_items[0] > 1,
            'right_padding': range_items[-1] < range_value,
        }

    range_items = range(current_item-padding, current_item+padding+1)
    return {
        'range_items': range_items,
        'left_padding': True,
        'right_padding': True,
    }


@register.assignment_tag(takes_context=True)
def is_context_variable(context, variable_name):
    """
    Returns true if the given variable name is in the template context.

    Usage::

        {% is_context_variable "variable_name" as variable_exists %}
        {% if variable_exists %}
            ...
        {% endif %}

    """
    return variable_name in context


@register.inclusion_tag('django_libs/analytics.html')
def render_analytics_code(anonymize_ip='anonymize'):
    """
    Renders the google analytics snippet.

    :anonymize_ip: Use to add/refuse the anonymizeIp setting.

    """
    return {
        'ANALYTICS_TRACKING_ID': getattr(
            settings, 'ANALYTICS_TRACKING_ID', 'UA-XXXXXXX-XX'),
        'anonymize_ip': anonymize_ip,
    }


@register.inclusion_tag('django_libs/analytics2.html')
def render_analytics2_code():
    """
    Renders the new google analytics snippet.

    """
    return {
        'ANALYTICS_TRACKING_ID': getattr(
            settings, 'ANALYTICS_TRACKING_ID', 'UA-XXXXXXX-XX'),
        'ANALYTICS_DOMAIN': getattr(
            settings, 'ANALYTICS_DOMAIN', 'auto')
    }


class VerbatimNode(template.Node):
    def __init__(self, text):
        self.text = text

    def render(self, context):
        return self.text


@register.simple_tag(takes_context=True)
def save(context, key, value):
    """
    Saves any value to the template context.

    Usage::

        {% save "MYVAR" 42 %}
        {{ MYVAR }}

    """
    context.dicts[0][key] = value
    return ''


@register.simple_tag(takes_context=True)
def sum(context, key, value, multiplier=1):
    """
    Adds the given value to the total value currently held in ``key``.

    Use the multiplier if you want to turn a positive value into a negative
    and actually substract from the current total sum.

    Usage::

        {% sum "MY_TOTAL" 42 -1 %}
        {{ MY_TOTAL }}

    """
    if key not in context.dicts[0]:
        context.dicts[0][key] = 0
    context.dicts[0][key] += value * multiplier
    return ''


@register.assignment_tag
def set_context(value):
    return value


@register.tag
def verbatim(parser, token):
    """Tag to render x-tmpl templates with Django template code."""
    text = []
    while 1:
        token = parser.tokens.pop(0)
        if token.contents == 'endverbatim':
            break
        if token.token_type == template.TOKEN_VAR:
            text.append('{{ ')
        elif token.token_type == template.TOKEN_BLOCK:
            text.append('{%')
        text.append(token.contents)
        if token.token_type == template.TOKEN_VAR:
            text.append(' }}')
        elif token.token_type == template.TOKEN_BLOCK:
            if not text[-1].startswith('='):
                text[-1:-1] = [' ']
            text.append(' %}')
    return VerbatimNode(''.join(text))


@register.filter
def exclude(qs, qs_to_exclude):
    """Tag to exclude a qs from another."""
    return qs.exclude(pk__in=qs_to_exclude.values_list('pk'))


@register.assignment_tag
def time_until(date_or_datetime):
    if isinstance(date_or_datetime, datetime.date):
        datetime_ = datetime.datetime(
            date_or_datetime.year,
            date_or_datetime.month,
            date_or_datetime.day,
            0, 0)
    else:
        datetime_ = date_or_datetime
    return datetime_ - datetime.datetime.now()


@register.assignment_tag
def days_until(date_or_datetime):
    days = time_until(date_or_datetime).days
    if days >= 0:
        return days
    return 0


@register.assignment_tag
def hours_until(date_or_datetime):
    closes_in = time_until(date_or_datetime)
    if closes_in.days < 0:
        return 0
    return closes_in.seconds / 3600


@register.assignment_tag
def minutes_until(date_or_datetime):
    closes_in = time_until(date_or_datetime)
    if closes_in.days < 0:
        return 0
    return closes_in.seconds / 60 - hours_until(date_or_datetime) * 60


@register.filter(is_safe=False)
@stringfilter
def append_s(value):
    """
    Adds the possessive s after a string.

    value = 'Hans' becomes Hans'
    and value = 'Susi' becomes Susi's

    """
    if value.endswith('s'):
        return u"{0}'".format(value)
    else:
        return u"{0}'s".format(value)


@register.assignment_tag
def get_site():
    return Site.objects.get_current()



"""Custom middlewares for the project."""
from __future__ import absolute_import
import re

from django.conf import settings
from django.http import HttpResponseRedirect

try:
    from django.middleware.common import _is_ignorable_404
except ImportError:
    _is_ignorable_404 = None


class AjaxRedirectMiddleware(object):
    """
    Middleware that sets a made up status code when a redirect has happened.

    This is necessary for AJAX calls with jQuery. It seems to set the status
    code to 200 when in reality it was a 301 or 302.

    If you want to override this behaviour for some of your ajax calls, you
    can add `ajax_redirect_passthrough` as a hidden field or as a GET
    parameter.

    """
    def process_response(self, request, response):
        if request.is_ajax():
            if request.REQUEST.get('ajax_redirect_passthrough'):
                return response
            if type(response) == HttpResponseRedirect:
                response.status_code = 278
        return response


class ErrorMiddleware(object):
    """Alter HttpRequest objects on Error."""

    def process_exception(self, request, exception):
        """
        Add user details.
        """
        if request.user and hasattr(request.user, 'email'):
            request.META['USER'] = request.user.email


class SSLRedirect:
    """
    Redirects all non-SSL requests to the SSL versions.

    You can add exceptions via the setting ``NO_SSL_URLS``. This allows you to
    forward your whole website to the SSL version except for a few URLs that
    you need to serve via non-SSL for whatever reason.

    """
    def process_request(self, request):
        no_ssl_urls = getattr(settings, 'NO_SSL_URLS', [])
        urls = tuple([re.compile(url) for url in no_ssl_urls])

        secure = False
        for url in urls:
            if not url.match(request.path):
                secure = True
                break
        if not secure == self._is_secure(request):
            return self._redirect(request, secure)

    def _is_secure(self, request):
        if request.is_secure():
            return True

        # Handle the Webfaction case until this gets resolved in the
        # request.is_secure()
        if 'HTTP_X_FORWARDED_SSL' in request.META:
            return request.META['HTTP_X_FORWARDED_SSL'] == 'on'

        return False

    def _redirect(self, request, secure):
        protocol = secure and "https" or "http"
        if secure:
            host = getattr(settings, 'SSL_HOST', request.get_host())
        else:
            host = getattr(settings, 'HTTP_HOST', request.get_host())
        newurl = "%s://%s%s" % (protocol, host, request.get_full_path())
        if settings.DEBUG and request.method == 'POST':
            raise Exception(
                "Django can't perform a SSL redirect while maintaining POST"
                " data. Please structure your views so that redirects only"
                " occur during GETs.")

        return HttpResponseRedirect(newurl)


if _is_ignorable_404:
    from .middleware_1_5 import *  # NOQA
else:
    from .middleware_1_6 import *  # NOQA



"""Custom test runner for the project."""
from django_coverage.coverage_runner import CoverageRunner
from django_nose import NoseTestSuiteRunner


class NoseCoverageTestRunner(CoverageRunner, NoseTestSuiteRunner):
    """
    Custom test runner that uses nose and coverage.

    *deprecated* No need to use this runner for Django>=1.7.

    """
    pass
